#ifndef __VIENNA_RNA_PACKAGE_READ_EPARS_H__
#define __VIENNA_RNA_PACKAGE_READ_EPARS_H__

/**
 *  \addtogroup energy_parameters_rw
 *  \brief  Read and Write energy parameter sets from and to text files
 *
 *  A default set of parameters, identical to the one described in
 *  \cite mathews:2004 and \cite turner:2010, is compiled into the library.
 *
 *  @{
 *  \file read_epars.h
 *
 */

/**
 *  \brief Identifiers for energy contribution parameters in parameter files
 *    
 *  DO NOT ALTER THEIR ORDER unless you are also reordering the char pointers
 *  in the const array 'identifier' in read_epars.c!!!
 */
enum parset {
  /* misc identifiers */
  UNKNOWN = -1,
  QUIT = 0, /* end of parameter file */

  /*
     The following enumeration identifiers are used throughout the library
     to indicate read/write from/to the regular energy contribution arrays,
     i.e. the ones used in all folding recursions for base pair types <= 7.
     For simplicity and to keep things clear we associate these arrays with
     RNA contributions althought they can be filled with contributions for
     any kind of interaction via the read_parameter_file() function
  */

  /* RNA loop type specific identifiers */
  S, S_H,
  HP, HP_H,
  B, B_H,
  IL, IL_H,
  INT11, INT11_H, INT21, INT21_H, INT22, INT22_H,
  ML,
  /* RNA mismatch identifiers */
  MMH, MMH_H,
  MMI, MMI_H,
  MMI1N, MMI1N_H, MMI23, MMI23_H,
  MMM, MMM_H,
  MME, MME_H,
  /* RNA dangling end identifiers */
  D5, D5_H,
  D3, D3_H,
  /* RNA special hairpin identifiers */
  TRI, TL, HEX,
  /* RNA other identifiers */
  NIN, MISC,

  /*
    Below are the identifiers used for DNA parameters in all the hybridisation
    routines. Note that they will be used only if RNA/DNA hybridisation is
    activated (base pair types >= 7 are occuring)
  */

  /* DNA loop type specific identifiers */
  S_DNA, S_H_DNA,
  HP_DNA, HP_H_DNA,
  B_DNA, B_H_DNA,
  IL_DNA, IL_H_DNA,
  INT11_DNA, INT11_H_DNA, INT21_DNA, INT21_H_DNA, INT22_DNA, INT22_H_DNA,
  ML_DNA,
  /* RNA mismatch identifiers */
  MMH_DNA, MMH_H_DNA,
  MMI_DNA, MMI_H_DNA,
  MMI1N_DNA, MMI1N_H_DNA, MMI23_DNA, MMI23_H_DNA,
  MMM_DNA, MMM_H_DNA,
  MME_DNA, MME_H_DNA,
  /* DNA dangling end identifiers */
  D5_DNA, D5_H_DNA,
  D3_DNA, D3_H_DNA,
  /* DNA special hairpin identifiers */
  TRI_DNA, TL_DNA, HEX_DNA,
  /* DNA other identifiers */
  NIN_DNA, MISC_DNA,

  /* Finally, the RNA/DNA hybrid identifiers */

  /* RNA/DNA hybrid loop type specific identifiers */
  S_HYBRID, S_H_HYBRID,
  HP_HYBRID, HP_H_HYBRID,
  B_HYBRID, B_H_HYBRID,
  IL_HYBRID, IL_H_HYBRID,
  INT11_HYBRID, INT11_H_HYBRID, INT21_HYBRID, INT21_H_HYBRID, INT22_HYBRID, INT22_H_HYBRID,
  ML_HYBRID,
  /* RNA/DNA hybrid mismatch identifiers */
  MMH_HYBRID, MMH_H_HYBRID,
  MMI_HYBRID, MMI_H_HYBRID,
  MMI1N_HYBRID, MMI1N_H_HYBRID, MMI23_HYBRID, MMI23_H_HYBRID,
  MMM_HYBRID, MMM_H_HYBRID,
  MME_HYBRID, MME_H_HYBRID,
  /* RNA/DNA hybrid dangling end identifiers */
  D5_HYBRID, D5_H_HYBRID,
  D3_HYBRID, D3_H_HYBRID,
  /* RNA/DNA hybrid special hairpin identifiers */
  TRI_HYBRID, TL_HYBRID, HEX_HYBRID,
  /* RNA/DNA hybrid other identifiers */
  NIN_HYBRID, MISC_HYBRID,
  
  /* This dummy must be the last entry in this enum! */
  DUMMY
};

#define VRNA_PARAMETER_FILE_RNA     1
#define VRNA_PARAMETER_FILE_DNA     2
#define VRNA_PARAMETER_FILE_HYBRID  4


/**
 *  \brief Read energy parameters from a file
 * 
 *  \param fname  The path to the file containing the energy parameters
 */
void  read_parameter_file(const char fname[]);

/**
 *  \brief Write energy parameters to a file
 * 
 *  \param fname  A filename (path) for the file where the current energy parameters will be written to
 */
void  write_parameter_file(const char fname[]);

void  write_parameter_file_full(const char fname[], int options);

/**
 *  \brief
 *
 */
enum  parset gettype(const char *ident);

/**
 *  \brief
 *
 */
const char  *settype(enum parset s);

/**
 *  @}
 */

#endif
